@@ -18,8 +18,6 @@
 #include "psupdate.h"
 
 #ifdef ELF_CAPABLE
-#define ELF_OBJECT 1
-#define ELF_FUNC 2
 
 #include <errno.h>
 #include <unistd.h>
@@ -115,7 +113,6 @@ make_elf_tbl (struct sym_s *p, int vartable)
   int i, type;
 
   type = vartable ? ELF_OBJECT : ELF_FUNC;
-
   /*
    * This is not really needed; it's here as a sanity check.  If the
    * values for various ELF internals are changed, we'll have other
@@ -147,3 +144,4 @@ read_elf_uts (int fd, struct new_utsname * uts, const char *syspath)
 }
 
 #endif /* ELF_CAPABLE */
+
