@@ -1,10 +1,14 @@
 /* The shadow of the original with only common prototypes now. */
+
+/* do not include the kernel inline definitions of string functions */
+#define _LINUX_STRING_H_ 
+
 #include <stdio.h>
 #include <sys/types.h>
 #include <linux/sched.h>
 
-char *wchan(unsigned int);
-char *find_func();
+char *wchan(unsigned long);
+char *find_func(unsigned long address);
 void *xcalloc(void *pointer, int size);
 void *xmalloc(unsigned int size);
 void *xrealloc(void *oldp, unsigned int size);
@@ -13,13 +17,16 @@ int   mult_lvl_cmp(void* a, void* b);
 int   node_mult_lvl_cmp(void* a, void* b);
 void  dump_keys(void);
        
-char *user_from_uid(int uid);
+char *user_from_uid(uid_t uid);
+char *group_from_gid(gid_t gid);
 
 int   open_sysmap(void);
 int   open_psdb(void);
 void  close_psdb(void);
 void  make_fnctbl(void);
 
+#if 0
 unsigned print_str    (FILE* file, char *s, unsigned max);
 unsigned print_strlist(FILE* file, char **strs, char* sep, unsigned max);
 unsigned snprint_strlist(char *buf, int max, char **strs, char *sep);
+#endif
