@@ -8,20 +8,12 @@
  *
  */
 
-/*
- * You can define ELF_CAPABLE at compile-time to get an a.out binary that can
- * handle ELF kernel images.
- */
-#if defined(__ELF__) && !defined(NO_ELF_CAPABILITY)
-# define ELF_CAPABLE
-# define ELF_OBJECT 1
-# define ELF_FUNC 2
-#endif
-
 #include <sys/types.h>
 #include <linux/utsname.h>
 
-#define        PSDATABASE      "/etc/psdatabase"
+#ifndef PSDATABASE
+#define        PSDATABASE      "/boot/psdatabase"
+#endif 
 
 struct dbtbl_s {
   off_t off;			/* offset in psdatabase */
