@@ -3,6 +3,22 @@
 
 #include "proc/psdata.h"
 
+/*
+ * You can define ELF_CAPABLE at compile-time to get an a.out binary that can
+ * handle ELF kernel images.
+ */
+#if defined(__ELF__) && !defined(NO_ELF_CAPABILITY)
+# define ELF_CAPABLE
+# define ELF_OBJECT 1
+# define ELF_FUNC 2
+#endif
+
+#if defined(__alpha__) || defined(__sparc__)
+# define BFD_CAPABLE
+#else
+# define AOUT_CAPABLE
+#endif
+
 #ifdef AOUT_CAPABLE
 extern int	check_aout_magic (int, const char *);
 extern int	read_aout_nlist (int, const char *);
