@@ -1,11 +1,11 @@
 #include "proc/whattime.h"
 #include "proc/version.h"
 
-int main(int argc, char *argv[]) {
+void main(int argc, char *argv[]) {
   set_linux_version();
   if(argc == 1)
     print_uptime();
   if((argc == 2) && (!strcmp(argv[1], "-V")))
     display_version();
-  return (0);
+  exit(0);
 }
