@@ -1,5 +1,5 @@
-/* w - show what logged in users are doing.  Almost entirely rewritten from
- * scratch by Charles Blake circa June 1996.  Some vestigal traces of the
+/* W - show what logged in users are doing.  Almost entirely rewritten from
+ * scratch by Charles Blake circal June 1996.  Some vestigal traces of the
  * original may exist.  That was done in 1993 by Larry Greenfield with some
  * fixes by Michael K. Johnson.
  */
@@ -8,10 +8,11 @@
 #include "proc/readproc.h"
 #include "proc/devname.h"
 #include "proc/ps.h"
-#include <errno.h>
+#include "proc/output.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <ctype.h>
+#include <errno.h>
 #include <string.h>
 #include <unistd.h>
 #include <time.h>
@@ -31,35 +32,44 @@ void    showinfo(utmp_t *login_rec, int formtype, int maxcmd, int from);
 proc_t *getproc(utmp_t *login_rec, char *tty, int *jcpu, int *bogus);
 void    print_logintime(time_t login_time, FILE* fout);
 
-#ifdef W_SHOWFROM
+#ifndef W_SHOWFROM
 #   define FROM_STRING "on"
 #else
 #   define FROM_STRING "off"
 #endif
 
+#ifdef NEW_TIME_DEFAULT
+int new_time = 1;
+#else
+int new_time = 0;
+#endif
+
 #define USERSZ (sizeof u->ut_user)
 
 int main(int argc, char **argv) {
-    int header=1, longform=1, from=1, args, maxcmd = 80, ch;
-    char *user = NULL;
-    utmp_t *u;
+    int header=1, longform=1, from=1, args, maxcmd = 80, fd;
+    char ch, *user = NULL;
+    utmp_t *u, *last;
+    struct stat sbuf;
     struct winsize win;
 
-#ifndef W_SHOWFROM
+#ifdef W_SHOWFROM
     from = 0;
 #endif
     set_linux_version();
 
-    for (args=0; (ch = getopt(argc, argv, "hlusfV")) != EOF; args++)
+    for (args=0; (ch = getopt(argc, argv, "ahlusfV")) != EOF; args++)
 	switch (ch) {
-	  case 'h': header = 0;		break;
-	  case 'l': longform = 1;	break;
-	  case 's': longform = 0;	break;
-	  case 'f': from = !from;	break;
+          case 'a': new_time = !new_time; break;
+	  case 'h': header = 0;		  break;
+	  case 'l': longform = 1;	  break;
+	  case 's': longform = 0;	  break;
+	  case 'f': from = !from;	  break;
 	  case 'V': display_version();	exit(0);
-	  case 'u': ignoreuser = 1;	break;
+	  case 'u': ignoreuser = 1;	  break;
 	  default:
-	    printf("usage: w -hlsufV [user]\n"
+	    printf("usage: w -ahlsufV [user]\n"
+                   "    -a    alternate time format\n"
 		   "    -h    skip header\n"
 		   "    -l    long listing (default)\n"
 		   "    -s    short listing\n"
@@ -69,8 +79,7 @@ int main(int argc, char **argv) {
 	    exit(1);
 	}
 
-    if ((argv[optind]))
-	user = (argv[optind]);
+    if (optind < argc) user = argv[optind];
 
     if (ioctl(1, TIOCGWINSZ, &win) != -1 && win.ws_col > 0)
 	maxcmd = win.ws_col;
@@ -82,75 +91,61 @@ int main(int argc, char **argv) {
     if (maxcmd < 3)
 	fprintf(stderr, "warning: screen width %d suboptimal.\n", win.ws_col);
 
-    procs = readproctab(PROC_FILLCMD|PROC_FILLTTY);
+    procs = readproctab(PROC_FILLCMD|PROC_FILLTTY|PROC_FILLSTAT);
 
     if (header) {				/* print uptime and headers */
 	print_uptime();
 	printf("USER     TTY      ");
 	if (from)
-	    printf("FROM            ");
+	    printf("FROM             ");
 	if (longform)
-	    printf("  LOGIN@   IDLE   JCPU   PCPU  WHAT\n");
+	    printf(" LOGIN@  IDLE   JCPU   PCPU  WHAT\n");
 	else
-	    printf("   IDLE  WHAT\n");
+	    printf("  IDLE   WHAT\n");
     }
-
-    utmpname(UTMP_FILE);
-    setutent();
-    while ((u=getutent())) {
- 	if (u->ut_type == USER_PROCESS &&
- 	    (user ? !strncmp(u->ut_user, user, USERSZ) : *u->ut_user))
- 	    showinfo(u, longform, maxcmd, from);
+    /* open + mmap utmp.  Sure I could us getutent, but I looked at
+     * the libc code and it didn't seem to add any real value. */
+    if ((fd = open(UTMP_FILE, O_RDONLY)) < 0 ||
+	fstat(fd, &sbuf) == -1 ||
+	(u = (utmp_t*)mmap(0, sbuf.st_size,
+                           PROT_READ, MAP_SHARED, fd, 0)) == (utmp_t*) -1 ||
+	close(fd) == -1)
+	return fprintf(stderr, "%s: %s during open,fstat,mmap,close\n",
+		       UTMP_FILE, sys_errlist[errno]);
+    if (u) {
+        for (last = u + sbuf.st_size / (sizeof *u); u <= last; u++)
+            if (u->ut_type == USER_PROCESS &&
+                (user ? !strncmp(u->ut_user, user, strlen(u->ut_user)) : *u->ut_user))
+                showinfo(u, longform, maxcmd, from);
     }
-    endutent();
-
     return 0;
 }
 
-/* This routine is careful since some programs leave utmp strings
- * unprintable.  Always outputs at least 16 chars padded with spaces
- * on the right if necessary.
+/* This routine is careful since some programs leave utmp strings unprintable.
+ * Always outputs `len' chars padded with spaces on the right if necessary.
  */
 void print_host(char* host, int len) {
-    char *last;
-    int width = 0;
-
-    /* FIXME: there should really be a way to configure this... */
-    /* for now, we'll just limit it to the 16 that the libc5 version
-     * of utmp uses.
-     */
-    if (len > 16) len = 16;
-    last = host + len;
-    for ( ; host < last ; host++)
-        if (isprint(*host) && *host != ' ') {
+    char *last = host + len - 1;
+    for ( ; host <= last ; host++)
+        if (isprint(*host) && *host != ' ' && !(*host & 0x80))
 	    fputc(*host, stdout);
-	    ++width;
-	} else
+	else
 	    break;
      /* if *any* unprintables(or blanks), replace rest of line with spaces */
-    while (width < 16) {
-	fputc(' ', stdout);
-	++width;
-    }
-}
-
-/* compact 7 char format for time intervals: should probably be in libproc */
-void print_time_ival7(time_t t, int centi_sec, FILE* fout) {
-    if (t >= 48*60*60)				/* > 2 days */
-	fprintf(fout, " %2ludays", t/(24*60*60));
-    else if (t >= 60*60)			/* > 1 hour */
-	fprintf(fout, " %2lu:%02um", t/(60*60), (unsigned) ((t/60)%60));
-    else if (t > 60)				/* > 1 minute */
-	fprintf(fout, " %2lu:%02u ", t/60, (unsigned) t%60);
-    else
-	fprintf(fout, " %2lu.%02us", t, centi_sec);
+     if (host <= last)
+	  for ( ; host <= last ; host++)
+	      fputc(' ', stdout);
 }
 
 time_t idletime(utmp_t *u, char *tty) {
     struct stat terminfo;
     if (stat(tty, &terminfo) != 0)
 	return 0;
-    return time(NULL) - terminfo.st_atime;
+    if (terminfo.st_atime > u->ut_time) {
+    	return time(NULL) - terminfo.st_atime;
+    } else {
+	return time(NULL) - u->ut_time;
+    }
 }
 
 void showinfo(utmp_t *u, int formtype, int maxcmd, int from) {
@@ -178,36 +173,54 @@ void showinfo(utmp_t *u, int formtype, int maxcmd, int from) {
     if (formtype) {
 	printf("%-9.8s%-9.8s", uname, u->ut_line);
 	if (from)
-	    print_host(u->ut_host, sizeof u->ut_host);
+	    print_host(u->ut_host, sizeof u->ut_host + 1);
 	print_logintime(u->ut_time, stdout);
 	if (*u->ut_line == ':')			/* idle unknown for xdm logins */
-	    printf("   ?   ");
-	else
-	    print_time_ival7(idletime(u, tty), 0, stdout);
-	print_time_ival7(jcpu/HZ, (jcpu%HZ)*(100./HZ), stdout);
+	    printf("       ");
+	else {
+            putc(' ',stdout);
+            if (new_time) {
+                print_time_ival(idletime(u, tty), 7, 0);
+            } else {
+                print_time_old_ival(idletime(u, tty), 7);
+            }
+            
+        }
+        putc(' ',stdout);
+        if (new_time){
+            print_time_ival(jcpu/HZ, 7, (jcpu%HZ) * 100 / HZ);
+        } else {
+            print_time_old_ival(jcpu/HZ, 7);
+        }
 	if (best) {
-	    int pcpu = best->utime + best->stime;
-	    print_time_ival7(pcpu/HZ, (pcpu%HZ)*(100./HZ), stdout);
+            int pcpu = best->utime + best->stime;
+	    putc(' ', stdout);
+            if (new_time){
+                print_time_ival(pcpu/HZ, 7, (pcpu%HZ) * 100 / HZ );
+            } else {
+                print_time_old_ival(pcpu/HZ, 7);
+            }
 	} else
 	    printf("   ?   ");
     } else {
 	printf("%-9.8s%-9.8s", u->ut_user, u->ut_line);
 	if (from)
-	    print_host(u->ut_host, sizeof u->ut_host);
-	if (*u->ut_line == ':')			/* idle unknown for xdm logins */
-	    printf("   ?   ");
-	else
-	    print_time_ival7(idletime(u, tty), 0, stdout);
+	    print_host(u->ut_host, sizeof u->ut_host + 1);
+        fputs(" ", stdout);
+        if (new_time){
+            print_time_ival(idletime(u, tty), 7, 0);
+        } else {
+            print_time_old_ival(idletime(u, tty), 7);
+        }
     }
-    fputs("  ", stdout);
-    if (best) {
+    fputs(" ", stdout);
+    if (best)
 	if (best->cmdline)
-	    print_strlist(stdout, best->cmdline, " ", maxcmd);
+	    print_strlist(best->cmdline, maxcmd, " ");
 	else
-	    printf("%*.*s", maxcmd, maxcmd, best->cmd);
-    } else {
+	    printf("-%-*.*s", maxcmd, maxcmd, best->cmd);
+    else
 	printf("-");
-    }
     fputc('\n', stdout);
 }
 
@@ -220,26 +233,22 @@ void print_logintime(time_t logt, FILE* fout) {
     struct tm *logtm, *curtm;
     int hour;
     char *merid; /* meridian indicator */
-    int today;
 
     curt = time(NULL);
     curtm = localtime(&curt);
-    /* localtime returns a pointer to static memory */
-    today = curtm->tm_yday;
     logtm = localtime(&logt);
     hour = logtm->tm_hour;
     merid = (hour < 12) ? "am" : "pm";
     if (hour >= 12) hour -= 12;
     if (hour == 0)  hour = 12;
-    if (curt - logt > 12*60*60 && logtm->tm_yday != today) {
+    if (curt - logt > 12*60*60 && logtm->tm_yday != curtm->tm_yday)
 	if (curt - logt > 6*24*60*60)
-	    fprintf(fout, " %2d%3s%2d", logtm->tm_mday, month[logtm->tm_mon],
+	    fprintf(fout, "%2d%3s%2d", logtm->tm_mday, month[logtm->tm_mon],
 		    logtm->tm_year % 100);
 	else
-	    fprintf(fout, " %3s%2d%s", weekday[logtm->tm_wday], hour, merid);
-    } else {
-	fprintf(fout, " %2d:%02d%s", hour, logtm->tm_min, merid);
-    }
+	    fprintf(fout, "%3s%2d%s", weekday[logtm->tm_wday], hour, merid);
+    else
+	fprintf(fout, "%2d:%02d%s", hour, logtm->tm_min, merid);
 }
 
 /* This function scans the process table accumulating total cpu times for
@@ -258,7 +267,7 @@ proc_t *getproc(utmp_t *u, char *tty, int *jcpu, int *found_utpid) {
 	    *found_utpid = 1;
 	if (!strcmp((**p).ttyc, line)) {
 	    (*jcpu) += (**p).utime + (**p).stime;
-	    if ((!ignoreuser || !strncmp((**p).user, u->ut_user, USERSZ)) &&
+	    if ((!ignoreuser || !strncmp((**p).user, u->ut_user, strlen(u->ut_user))) &&
 		((**p).pid == (**p).tpgid && (!best || (**p).pid > best->pid)))
 		best = *p;
 	}
