@@ -4,6 +4,7 @@
  *
  * Author:  Hans-Helmut Buehmann 20. Jan. 1996
  * 
+ * Modified for more recent kernels Helmut Geyer Oct. 1996
  */
 
 #include <X11/Xos.h>
@@ -31,14 +32,15 @@ void GetMemLoadPoint( w, closure, call_data )	/* Linux version */
 	    perror("xmem : trouble with /proc/meminfo ");
 	    exit(1);
 	} 
-        /* code + shared + buffer + free == 1.0 */
+        /* code + cached + buffer + free == 1.0 */
+	/* the old formula using shared doesn't work any longer */
 	ret.code
 	    = (double)(  mem[meminfo_main][meminfo_used] 
-		       - mem[meminfo_main][meminfo_shared]
-		       - mem[meminfo_main][meminfo_buffers])
+		       - mem[meminfo_main][meminfo_buffers]
+		       - mem[meminfo_main][meminfo_cached])
 	    / (double)mem[meminfo_main][meminfo_total];
-	ret.shared 
-	    = (double)mem[meminfo_main][meminfo_shared] 
+	ret.cached 
+	    = (double)mem[meminfo_main][meminfo_cached] 
 	    / (double)mem[meminfo_main][meminfo_total];
 	ret.buffer 
 	    = (double)mem[meminfo_main][meminfo_buffers] 
