@@ -75,7 +75,7 @@ static XrmOptionDescRec options_mem[] = {
  {"-hl",		"*mem.highlight",	XrmoptionSepArg,	NULL},
  {"-highlight",		"*mem.highlight",	XrmoptionSepArg,	NULL},
  {"-codecolor",		"*mem.codecolor",       XrmoptionSepArg,	NULL},
- {"-sharedcolor",	"*mem.sharedcolor",	XrmoptionSepArg,	NULL},
+ {"-cahedcolor",	"*mem.cachedcolor",	XrmoptionSepArg,	NULL},
  {"-buffercolor",	"*mem.buffercolor",	XrmoptionSepArg,	NULL},
  {"-freecolor",		"*mem.freecolor",	XrmoptionSepArg,	NULL},
  {"-swapcolor",		"*mem.swapcolor",	XrmoptionSepArg,	NULL},
@@ -130,8 +130,6 @@ void usage()
       "    -geometry geom          size and location of window\n");
     fprintf (stderr, 
       "    -fn font                font to use in label\n");
-    fprintf (stderr, 
-      "    -scale number           minimum number of scale lines\n");
     fprintf (stderr, 
       "    -update seconds         interval between updates\n");
     fprintf (stderr,
@@ -145,6 +143,8 @@ void usage()
     else {
       fprintf (stderr, 
       "    -fg color               graph and textcolor\n");
+      fprintf (stderr,
+      "    -scale number           minimum number of scale lines\n");
     }  
     fprintf (stderr, 
       "    -hl color               scale color\n");
@@ -152,7 +152,7 @@ void usage()
        fprintf (stderr, 
       "    -codecolor color        used code and stack memory color\n");
        fprintf (stderr, 
-      "    -shared color           used shared memory color\n");
+      "    -cachedcolor color      used cached memory color\n");
        fprintf (stderr, 
       "    -buffercolor color      used buffer memory color\n");
        fprintf (stderr, 
@@ -166,7 +166,7 @@ void usage()
       "    -jumpscroll value       number of pixels to scroll on overflow\n");
     if (ProgramType == MEMORY) {
        fprintf (stderr,
-      "One reference line refers to the avaible installed ram\n");   
+      "The reference line refers to the avaible installed ram\n");   
     }
     fprintf (stderr, "\n");
     exit(1);
